
/*

  code to deal with rom reading

*/
 
#include <stdio.h>
#include <string.h>
#include <windows.h>

#include <string>

#include "romspace.h"
#include "romlist.h"
#include "gui.h"

extern tGui* gui;


// constructor

tRomSpace::tRomSpace()
{
}

// destructor

tRomSpace::~tRomSpace()
{
}

// generic error in rom reading

void tRomSpace::readError(char* zipfile,char* romfile)
{
	char errmsg[1024];

	if (romfile==0)
	{
		sprintf(errmsg,". Unable to open %s rom.",zipfile);
	}
	else
	{
		sprintf(errmsg,". Unable open %s rom in %s zip.",romfile,zipfile);
	}

	gui->printMessage(errmsg);
}

// zipper reader

int tRomSpace::loadZippedRom(char* zipfile,
							 char* romfile,
							 unsigned long int size,
							 unsigned long int crc32,
							 unsigned char* dest)
{
	int err;
	unzFile uf=NULL;

	// open zipfile

	uf = unzOpen(zipfile);
	if (uf==NULL)
	{
		readError(zipfile,0);
		return 1;
	}

	err=unzLocateFile(uf,romfile,0);
	if (err!=UNZ_OK)
	{
		// if an error was found, try to open the file
		// searching it by its CRC32

		int blowout=0;

		if (unzGoToFirstFile(uf)!=UNZ_OK)
		{
			readError(zipfile,0);
			return 1;
		}

		while (!blowout)
		{
			unz_file_info ufi;
			unzGetCurrentFileInfo(uf,&ufi,0,0,0,0,0,0);

			if (ufi.crc==crc32)
			{
				if (ufi.uncompressed_size==size)
				{
					// ok, or it is him, or it is
					// his secret brotha that he doesn't see by years
					blowout=1;
				}
			}
			else
			{
				if (unzGoToNextFile(uf)!=UNZ_OK)
				{
					readError(zipfile,0);
					return 1;
				}
			}
		}
	}

	err=unzOpenCurrentFile(uf);
	if (err!=UNZ_OK)
	{
		readError(zipfile,romfile);
		return 1;
	}

	err=unzReadCurrentFile(uf,dest,size);
	if (err!=size)
	{
		readError(zipfile,romfile);
		return 1;
	}

	unzCloseCurrentFile(uf);

	// close zipfile

	unzClose(uf);

	return 0;
}

// find romset

int tRomSpace::romsetByName(char* romsetname,int* rs)
{
	int found;
	int pos;

	found=0;
	pos=0;

	while ( (found==0) && (strlen(romsetList[pos].name)!=0) )
	{
		if (!strcmp(romsetname,romsetList[pos].name))
		{
			found=1;
		}
		else
		{
			pos++;
		}
	}

	if (found==0)
	{
		gui->printMessage(". Romset not supported");
		return 1;
	}

	*rs=pos;
	return 0;
}

// get graphic rom size

unsigned long int tRomSpace::getGromSize(tRomSet* rsp)
{
	int i;
	unsigned long int max=0;

	for (i=rsp->ncoderoms;i<(rsp->ncoderoms+rsp->ngfxroms);i++)
	{
		if (rsp->romlist[i].size>max)
		{
			max=rsp->romlist[i].size;
		}
	}

	return (max*rsp->ngfxroms);
}


// read roms

int tRomSpace::readRoms(char* path,char* romsetname,tCharset** chs)
{
	return 0;
}

/*

  functions to decode the charset

*/

int tRomSpace::readbit(const unsigned char *src,int bitnum)
{
	return (src[bitnum / 8] >> (7 - bitnum % 8)) & 1;
}

void tRomSpace::decodeCharset(tCharset* chset)
{
}